@@ -45,14 +45,18 @@
 #' ## Link to file - No text to display
 #' writeFormula(wb, "Sheet1", startRow = 4
 #'  , x = makeHyperlinkString(sheet = "testing", row = 3, col = 10
-#'    , file = system.file("loadExample.xlsx", package = "openxlsx")))
+#'    , file = system.file("extdata","loadExample.xlsx", package = "openxlsx")))
 #' 
 #' ## Link to file - Text to display
 #' writeFormula(wb, "Sheet1", startRow = 3
 #'   , x = makeHyperlinkString(sheet = "testing", row = 3, col = 10
-#'     , file = system.file("loadExample.xlsx", package = "openxlsx"), text = "Link to File."))
+#'     , file = system.file("extdata",loadExample.xlsx", package = "openxlsx"), text = "Link to File."))
+#' 
+#' \dontrun{
+#' saveWorkbook(wb, "internalHyperlinks.xlsx",overwrite=TRUE)
+#' }
+#' 
 #' 
-#' saveWorkbook(wb, "internalHyperlinks.xlsx")
 makeHyperlinkString <- function(sheet, row = 1, col = 1, text = NULL, file = NULL){
   
   od <- getOption("OutDec")
@@ -103,7 +107,7 @@ classStyles <- function(wb, sheet, startRow, startCol, colNames, nRow, colClasse
     
     ## style hyperlinks
     inds <- which(sapply(colClasses, function(x) "hyperlink" %in% x))
-
+    
     hyperlinkstyle <- createStyle(textDecoration = "underline")
     hyperlinkstyle$fontColour <- list("theme"="10")
     styleElements <- list("style" = hyperlinkstyle,
@@ -177,14 +181,14 @@ classStyles <- function(wb, sheet, startRow, startCol, colNames, nRow, colClasse
                           "sheet" =  wb$sheet_names[sheet],
                           "rows" = rep.int(rowInds, times = length(inds)),
                           "cols" = rep(inds + startCol, each = length(rowInds)))
-
+    
     newStylesElements <- append(newStylesElements, list(styleElements))
   }
   
   ## style big mark
   if("scientific" %in% allColClasses){
     inds <- which(sapply(colClasses, function(x) "scientific" %in% x))
-
+    
     styleElements <- list("style" = createStyle(numFmt = "scientific"),
                           "sheet" =  wb$sheet_names[sheet],
                           "rows" = rep.int(rowInds, times = length(inds)),
@@ -196,7 +200,7 @@ classStyles <- function(wb, sheet, startRow, startCol, colNames, nRow, colClasse
   ## style big mark
   if("3" %in% allColClasses | "comma" %in% allColClasses){
     inds <- which(sapply(colClasses, function(x) "3" %in% tolower(x) | "comma" %in% tolower(x)))
-
+    
     styleElements <- list("style" = createStyle(numFmt = "3"),
                           "sheet" =  wb$sheet_names[sheet],
                           "rows" = rep.int(rowInds, times = length(inds)),
@@ -235,7 +239,7 @@ classStyles <- function(wb, sheet, startRow, startCol, colNames, nRow, colClasse
     }
     
   }
-    
+  
   
   
   invisible(1)
@@ -307,7 +311,7 @@ writeCommentXML <- function(comment_list, file_name){
   xml <- '<comments xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">'
   xml <- c(xml, paste0('<authors>', paste(sprintf('<author>%s</author>', authors), collapse = ""), '</authors><commentList>'))
   
-
+  
   for(i in 1:length(comment_list)){
     
     authorInd <- which(authors == comment_list[[i]]$author) - 1L
@@ -327,6 +331,9 @@ writeCommentXML <- function(comment_list, file_name){
 }
 
 
+illegalchars <- c('&', '"', "'", '<', '>', "\a", "\b", "\v", "\f")
+illegalcharsreplace <- c("&amp;", "&quot;", "&apos;", "&lt;", "&gt;", "", "", "", "")
+
 
 replaceIllegalCharacters <- function(v){
   
@@ -335,20 +342,10 @@ replaceIllegalCharacters <- function(v){
   
   flg <- vEnc != "UTF-8"
   if(any(flg))
-    v[flg] <- iconv(v[flg], from = "", to = "UTF-8")
+    v[flg] <- stri_conv(v[flg], from = "", to = "UTF-8")
   
-  v <- gsub('&', "&amp;", v, fixed = TRUE)
-  v <- gsub('"', "&quot;", v, fixed = TRUE)
-  v <- gsub("'", "&apos;", v, fixed = TRUE)
-  v <- gsub('<', "&lt;", v, fixed = TRUE)
-  v <- gsub('>', "&gt;", v, fixed = TRUE)
+  v <- stri_replace_all_fixed(v, illegalchars, illegalcharsreplace, vectorize_all = FALSE)
   
-  ## Escape sequences
-  v <- gsub("\a", "", v, fixed = TRUE)
-  v <- gsub("\b", "", v, fixed = TRUE)
-  v <- gsub("\v", "", v, fixed = TRUE)
-  v <- gsub("\f", "", v, fixed = TRUE)
-
   return(v)
 }
 
@@ -512,15 +509,20 @@ get_named_regions_from_string <- function(dn){
   dn <- gsub("</workbook>", "", dn, fixed = TRUE)
   
   dn <- unique(unlist(strsplit(dn, split = "</definedName>", fixed = TRUE)))
+  dn <- dn[grepl("<definedName", dn, fixed=TRUE)]
   
   dn_names <- regmatches(dn, regexpr('(?<=name=")[^"]+', dn, perl = TRUE))
   
   dn_pos <- regmatches(dn, regexpr("(?<=>).*", dn, perl = TRUE))
-  dn_coords <- regmatches(dn_pos, regexpr("(?<=!).*", dn_pos, perl = TRUE))
-  dn_coords <- gsub("$", "", dn_coords, fixed = TRUE)
+  dn_pos <- gsub("[$']", "", dn_pos)
   
-  dn_sheets <- regmatches(dn_pos, regexpr(".*(?=!)", dn_pos, perl = TRUE))
-  dn_sheets <- gsub("'", "", dn_sheets, fixed = TRUE)
+  has_bang <- grepl("!", dn_pos, fixed=TRUE)
+  dn_sheets <- ifelse(has_bang,
+                      gsub("^(.*)!.*$", "\\1", dn_pos),
+                      "")
+  dn_coords <- ifelse(has_bang,
+                      gsub("^.*!(.*)$", "\\1", dn_pos),
+                      "")
   
   attr(dn_names, "sheet") <- dn_sheets
   attr(dn_names, "position") <- dn_coords
@@ -582,7 +584,7 @@ buildBorder <- function(x){
     if(length(tmp) == 1)
       sideBorder[[i]] <- tmp
   }
-
+  
   sideBorder <- sideBorder[sideBorder != ""]
   x <- x[sideBorder != ""]
   if(length(sideBorder) == 0)
@@ -768,9 +770,9 @@ getSharedStringsFromFile <- function(sharedStringsFile, isFile){
 }
 
 
-clean_names <- function(x){
+clean_names <- function(x,schar){
   x <- gsub("^[[:space:]]+|[[:space:]]+$", "", x)
-  x <- gsub("[[:space:]]+", ".", x)
+  x <- gsub("[[:space:]]+", schar, x)
   return(x)
 }
 
@@ -861,3 +863,23 @@ getFile <- function(xlsxFile){
   
 }
 
+# Rotate the 15-bit integer by n bits to the 
+hashPassword <- function(password) {
+  # password limited to 15 characters
+  chars = head(strsplit(password, "")[[1]], 15)
+  # See OpenOffice's documentation of the Excel format: http://www.openoffice.org/sc/excelfileformat.pdf
+  # Start from the last character and for each character
+  # - XOR hash with the ASCII character code
+  # - rotate hash (16 bits) one bit to the left
+  # Finally, XOR hash with 0xCE4B and XOR with password length
+  # Output as hex (uppercase)
+  rotate16bit <- function(hash, n = 1) {
+    bitwOr(bitwAnd(bitwShiftR(hash, 15 - n), 0x01), bitwAnd(bitwShiftL(hash, n), 0x7fff));
+  }
+  hash = Reduce(function(char, h) {
+    h = bitwXor(h, as.integer(charToRaw(char)))
+    rotate16bit(h, 1)
+  }, chars, 0, right = TRUE)
+  hash = bitwXor(bitwXor(hash, length(chars)), 0xCE4B)
+  format(as.hexmode(hash), upper.case = TRUE)
+}
\ No newline at end of file
