@@ -1,6 +1,7 @@
 #' @name writeData
 #' @title Write an object to a worksheet
 #' @author Alexander Walker
+#' @import stringi
 #' @description Write an object to worksheet with optional styling.
 #' @param wb A Workbook object containing a worksheet.
 #' @param sheet The worksheet to write to. Can be the worksheet index or name.
@@ -38,7 +39,8 @@
 #'    \item{\bold{slantDashDot}}{ slanted dash-dot border}
 #'   }
 #' @param withFilter If \code{TRUE}, add filters to the column name row. NOTE can only have one filter per worksheet. 
-#' @param keepNA If \code{TRUE}, NA values are converted to #N/A in Excel else NA cells will be empty.
+#' @param keepNA If \code{TRUE}, NA values are converted to #N/A (or \code{na.string}, if not NULL) in Excel, else NA cells will be empty.
+#' @param na.string If not NULL, and if \code{keepNA} is \code{TRUE}, NA values are converted to this string in Excel.
 #' @param name If not NULL, a named region is defined.
 #' @param sep Only applies to list columns. The separator used to collapse list columns to a character vector e.g. sapply(x$list_column, paste, collapse = sep).
 #' @seealso \code{\link{writeDataTable}}
@@ -112,7 +114,7 @@
 #' ## - vectors/columns with class 'hyperlink' are written as hyperlinks'
 #' 
 #' v <- rep("https://CRAN.R-project.org/", 4)
-#' names(v) <- paste("Hyperlink", 1:4) # Optional: names will be used as display text
+#' names(v) <- paste0("Hyperlink", 1:4) # Optional: names will be used as display text
 #' class(v) <- 'hyperlink'
 #' writeData(wb, "Cars", x = v, xy = c("B", 32))
 #'
@@ -122,7 +124,7 @@
 #' ## - vectors/columns with class 'formula' are written as formulas'
 #' 
 #' df <- data.frame(x=1:3, y = 1:3,
-#'                  z = paste(paste0("A", 1:3+1L), paste0("B", 1:3+1L), sep = " + "),
+#'                  z = paste0(paste0("A", 1:3+1L), paste0("B", 1:3+1L), sep = " + "),
 #'                  stringsAsFactors = FALSE)
 #' 
 #' class(df$z) <- c(class(df$z), "formula")
@@ -134,7 +136,7 @@
 #' ## Save workbook
 #' ## Open in excel without saving file: openXL(wb)
 #' 
-#' saveWorkbook(wb, "writeDataExample.xlsx", overwrite = TRUE)
+#' \dontrun{saveWorkbook(wb, "writeDataExample.xlsx", overwrite = TRUE)}
 writeData <- function(wb, 
                       sheet,
                       x,
@@ -149,6 +151,7 @@ writeData <- function(wb,
                       borderStyle = getOption("openxlsx.borderStyle", "thin"),
                       withFilter = FALSE,
                       keepNA = FALSE,
+                      na.string = NULL,
                       name = NULL,
                       sep = ", "){
   
@@ -267,12 +270,12 @@ writeData <- function(wb,
   if(withFilter){
     
     coords <- data.frame("x" = c(startRow, startRow + nRow + colNames - 1L), "y" = c(startCol, startCol + nCol - 1L))
-    ref <- paste(getCellRefs(coords), collapse = ":")
+    ref <- stri_join(getCellRefs(coords), collapse = ":")
     
     wb$worksheets[[sheetX]]$autoFilter <- sprintf('<autoFilter ref="%s"/>', ref)
 
     l <- convert_to_excel_ref(cols = unlist(coords[,2]), LETTERS = LETTERS)
-    dfn <- sprintf("'%s'!%s", names(wb)[sheetX],  paste0("$", l, "$", coords[,1], collapse=":"))
+    dfn <- sprintf("'%s'!%s", names(wb)[sheetX],  stri_join("$", l, "$", coords[,1], collapse=":"))
     
     dn <- sprintf('<definedName name="_xlnm._FilterDatabase" localSheetId="%s" hidden="1">%s</definedName>', sheetX - 1L, dfn)
     
@@ -298,6 +301,7 @@ writeData <- function(wb,
                colClasses = colClasss2,
                hlinkNames = hlinkNames,
                keepNA = keepNA,
+               na.string = na.string,
                list_sep = sep)
   
   ## header style  
@@ -314,8 +318,8 @@ writeData <- function(wb,
   ## named region
   if(!is.null(name)){
     
-    ref1 <- paste0("$", convert_to_excel_ref(cols = startCol, LETTERS = LETTERS), "$", startRow)
-    ref2 <- paste0("$", convert_to_excel_ref(cols = startCol + nCol - 1L, LETTERS = LETTERS), "$", startRow + nRow - 1L + colNames)
+    ref1 <- stri_join("$", convert_to_excel_ref(cols = startCol, LETTERS = LETTERS), "$", startRow)
+    ref2 <- stri_join("$", convert_to_excel_ref(cols = startCol + nCol - 1L, LETTERS = LETTERS), "$", startRow + nRow - 1L + colNames)
     wb$createNamedRegion(ref1 = ref1, ref2 = ref2, name = name, sheet = wb$sheet_names[wb$validateSheet(sheet)])
     
   }
@@ -441,7 +445,7 @@ writeData <- function(wb,
 #' writeData(wb, sheet = 2, x = v2, startCol = 10, startRow = 2)
 #' 
 #' ## Save workbook
-#' saveWorkbook(wb, "writeFormulaExample.xlsx", overwrite = TRUE)
+#' \dontrun{saveWorkbook(wb, "writeFormulaExample.xlsx", overwrite = TRUE)}
 #' 
 #' 
 #' ## Writing internal hyperlinks
@@ -449,7 +453,7 @@ writeData <- function(wb,
 #' addWorksheet(wb, "Sheet1")
 #' addWorksheet(wb, "Sheet2")
 #' writeFormula(wb, "Sheet1", x = '=HYPERLINK("#Sheet2!B3", "Text to Display - Link to Sheet2")')
-#' saveWorkbook(wb, "writeFormulaHyperlinkExample.xlsx", overwrite = TRUE)
+#' \dontrun{saveWorkbook(wb, "writeFormulaHyperlinkExample.xlsx", overwrite = TRUE)}
 #' 
 writeFormula <- function(wb, 
                          sheet,
